/*
        pravidla:

		1s=1000ms
		Uint32 SDL_GetTicks() vracia pocet ms od startu SDL,
		po zhruba 49 dnoch dojde k prekroceniu 32bit limitu
		a citac sa vynuluje
	uzitocne vzorce:

		rychlost		v=s/t [m/s]
					v=at [m/s]
		zrychlenie		a=s/t2 [m/s2]
		gravitacne zrychlenie	g=v/t [m/s2], g na Zemi=9.81 m/s2
					v=gt
*/
// hlavickove subory *******************************************************
#include <math.h>
// GL a GLU kniznica
#include <GL/gl.h>
#include <GL/glu.h>
// funkcie SDL kniznice
#include <SDL/SDL.h>

// .h subor
#include "mCamera.h"

// konstruktor *************************************************************
mCamera::mCamera()
{
 scr_w=0.0f; scr_h=0.0f; // pociatocne rozmery okna

 cam_reset(); // reset parametrov kamery

 collision_onoff=1; // detekcia kolizii je zapnuta
}

// destruktor **************************************************************
mCamera::~mCamera()
{

}

// reset pohladu kamery ****************************************************
void mCamera::cam_reset(void)
{
 // naplnenie matice kamery pociatocnymi hodnotami
 for (t_a=0;t_a<16;t_a++) matrix[t_a]=0.0f;
 matrix[0]=1.0f; matrix[5]=1.0f; matrix[10]=1.0f; matrix[15]=1.0f;
 // nastavenie parametrov kamery
 posx=0.0f; posy=-5.0f; posz=-25.0f;
 movx=0.0f; movy=0.0f; movz=0.0f;
 angx=0.0f; angy=0.0f; angz=0.0f;
 rotx=0.0f; roty=0.0f; rotz=0.0f;
 oldx=0.0f; oldy=0.0f; oldz=0.0f;
 targetx=0.0f; targety=0.0f; targetz=0.0f;
 speed=20.0f; slide=20.0f; alt=20.0f;
 acceleration=2.0f; gravity=9.81f;
 viewdist=5000.0f;
 FOV=60.0f;
 JumpPower=0.0f;

 // prepocet pre posun
 go_x=0.0f; go_y=0.0f; go_z=0.0f;

 // nastavenie faktorov pre prepocet posunu a rotacie
 rotx_factor=4.0f; roty_factor=4.0f; rotz_factor=32.0f;
 movx_factor=2.0f; movy_factor=2.0f; movz_factor=2.0f;

 // pociatocne zresetovanie posunu
 goForward=0; goBack=0; goUp=0; goDown=0; goRight=0; goLeft=0; goFast=0;
 goJump=0;

 // strelba
 isFire1=0;
 isFire2=0;

 // sposob vypoctu pohybu
 movetype=1;

 // prvotne nacitenie casoveho okamihu
 oldTime=SDL_GetTicks(); actTime=oldTime;
 difTime=0.0f;
 gravityTime=0.0f;

 ogl_fps=0.0f;
 ogl_frames=0.0f;
}

// reinicializacia kamery **************************************************
void mCamera::cam_reinit(GLfloat w,GLfloat h)
{
 glMatrixMode(GL_PROJECTION); // vyber projekcnej matice
 glLoadIdentity();            // zresetovanie projekcnej matice

 if (h==0.0f) h=1.0f;
 gluPerspective(FOV,w/h,1.0f,viewdist); // nastavenie perspektivneho premietania
 // FOV - zorny uhol kamery, w/h - aspect je pomer vysky a sirky zobrazovanej plochy
 // 1.0f - minimalna viditelna vzdialenost, viewdist - maximalna viditelna vzdialenost
 glViewport(0,0,(GLuint) w,(GLuint) h);  // nastavenie viditelnej plochy

 // TRANSFORMUJEME SCENU NIE KAMERU!
 glMatrixMode(GL_MODELVIEW); // vyber modelovej matice
 glLoadIdentity();           // zresetovanie modelovej matice

 cam_update(); // uprava parametrov kamery

 scr_w=w; scr_h=h; // ulozenie rozmerov okna
}

// aktualizacia nastaveni kamery *******************************************
void mCamera::cam_update(void)
{
 // ulozenie povodnej polohy
 oldx=posx; oldy=posy; oldz=posz;

 // SPECIALNY PRISTUP K ROTACII A NASLEDNEMU POSUNU KAMERY!
 glMatrixMode(GL_MODELVIEW);
 glLoadMatrixf(matrix);

 switch (movetype) {
  case 0:  // chodza
   // ROTACIA
   glRotatef(rotx,matrix[0],matrix[4],matrix[8]);
   glRotatef(roty,0.0f,1.0f,0.0f);
   // nastavenie noveho otocenia
   angx+=rotx; angy+=roty; angz+=rotz;

   glGetFloatv(GL_MODELVIEW_MATRIX,matrix);

   // nastavenie novej polohy
   posx+=matrix[0]*movx+0.0f*movy-matrix[2]*movz;
   posy+=matrix[4]*movx+1.0f*movy-matrix[6]*movz;
   posz+=matrix[8]*movx+0.0f*movy-matrix[10]*movz;
   break;
  case 1:  // let
   // ROTACIA
   glRotatef(rotx,matrix[0],matrix[4],matrix[8]);
   glRotatef(roty,matrix[1],matrix[5],matrix[9]);
   glRotatef(rotz,matrix[2],matrix[6],matrix[10]);
   // nastavenie noveho otocenia
   angx+=rotx; angy+=roty; angz+=rotz;

   glGetFloatv(GL_MODELVIEW_MATRIX,matrix);

   // nastavenie novej polohy
   posx+=matrix[0]*movx+matrix[1]*movy-matrix[2]*movz;
   posy+=matrix[4]*movx+matrix[5]*movy-matrix[6]*movz;
   posz+=matrix[8]*movx+matrix[9]*movy-matrix[10]*movz;
   break;
  default: // dalsie typy pohybu...
   break;
 }

 // nastavenie smeroveho vektora kamery
 targetx=matrix[2];
 targety=matrix[6];
 targetz=matrix[10];
}

// posun kamery ************************************************************
void mCamera::cam_move(void)
{
 // cas...
 actTime=SDL_GetTicks();
 difTime=((GLfloat) (actTime-oldTime))/1000.0f;
 oldTime=actTime;

 // INFO: vypis casoveho okamihu uplynuteho medzi vykreslenim dvoch scen
 //fprintf(stderr,"%f\n",difTime);

 // pocitadlo FPS
 ogl_frames++;
 if (difTime>=0.001f) {
  ogl_fps=ogl_frames/difTime;
  ogl_frames=0.0f;
 }

 if (goForward) go_z=speed*difTime;    // dopredu a dozadu
 else if (goBack) go_z=-speed*difTime;
      else go_z=0.0f;

 if (goRight) go_x=slide*difTime;      // vlavo a vpravo
 else if (goLeft) go_x=-slide*difTime;
      else go_x=0.0f;

 if (goUp) go_y=alt*difTime;           // hore a dole
 else if (goDown) go_y=-alt*difTime;
      else {
       if (gravityTime==0.0f) { // ak je avatar na zemi
        if (goJump) {           // ak bola stlaceny klavesa pre skok
         if (ogl_fps!=0.0f) JumpPower=sqrt(100.0f/ogl_fps); // vypocet sily skoku (zavisle od FPS!)
         else JumpPower=sqrt(100.0f);                       // kontrola na FPS=0!
        }
        else JumpPower=0.0f;    // inac nie je skok
       }
       // zvacsovanie sa casu trvania skoku/posobenia gravitacie
       gravityTime+=difTime;
       // vypocet klesania v zavislosti od skoku a gravitacie
       if (movetype==0) go_y=JumpPower-gravity*gravityTime/2.0f;
       else go_y=0.0f;
       // klesanie nemoze byt vacsie ako rozmer avatara, lebo nastanu chyby pri vypocte kolizie
       if (go_y<-1.0f) go_y=-1.0f;
      }

 if (goFast) { go_x*=10.0f; go_y*=1.0f; go_z*=10.0f; }

 movx=-go_x*movx_factor;
 movy=-go_y*movy_factor;
 movz=-go_z*movz_factor;

 // zrusenie dalsieho mozneho posunu, rotacie alebo skoku
 goForward=0; goBack=0; goUp=0; goDown=0; goRight=0; goLeft=0; goFast=0;
 goJump=0;
}

// rotacia kamery **********************************************************
void mCamera::cam_rotate(GLfloat x,GLfloat y,GLfloat z)
{
 rotx=x/rotx_factor; roty=y/roty_factor; rotz=z/rotz_factor;
}

// zap/vyp detekcie kolizii ************************************************
void mCamera::CollisionOnOff(void)
{
 if (collision_onoff==0) collision_onoff=1; else collision_onoff=0;

 SDL_Delay(200);
}
